# JWST General Target Visibility Tool (jwst_gtvt)

JWST requires shielding from the Sun for operation, which limits the available position angles observable at a given time. This script calculates the allowed position angles for a given Right Ascension and Declination for each instrument on the telescope. To report any issues, please use the JWST help portal https://jwsthelp.stsci.edu or feel free to open a github issue https://github.com/spacetelescope/jwst_gtvt/issues.

# Dependencies

This tool requires a few packages, all of which are included in the Anaconda Python distribution:

* astropy

* astroquery (for moving target support)

* docopt

* maplotlib

* numpy

* pysiaf

* pandas

* tabulate

# Installation

You can install the tool using `pip` with 
`pip install jwst-gtvt`

Alternatively, you can clone the respository from GitHub and install the tool from inside the resulting directory with
`pip install -e .`

We also provide conda environments:

    $ conda env create -f environment_python_3.10.yml 
    $ conda activate jwst-gtvt-3.10'
    $ pip install -e .

# Usage

There are two scripts available: `jwst_gtvt` for fixed targets and `jwst_mtvt` for moving targets. To see the help info use:

    $ jwst_gtvt --help
    Usage:
        jwst_gtvt --ra=<ra> --dec=<dec> [--start_date=<obs_start>] [--end_date=<obs_end>] [--instrument=<inst>] [--target_name=<name>] [--write_ephemeris=<write_path>] [--write_plot=<plot_path>] [--silent]

    Arguments:
        --ra=<ra>    Right ascension of target to observe with JWST.
        --dec=<dec>   Declination of target to observe with JWST.

        Options:
        [--start_date]         Start date for plot
        [--end_date]           End date for plot
        [--instrument]         JWST instrument to plot individually
        [--target_name]        User provided name for target
        [--write_ephemeris]    File name to write ephemeris to
        [--write_plot]         File name to write plot out to
        [--silent]             Boolean to print results to screen [default: False]
        -h --help              Show this screen.
        --version              Show version.


# Example

By default you need only specify R.A. and Dec. in either sexigesimal or degrees.
The observability windows will be printed to the terminal and a plot showing the windows for each instrument will pop up.

`$ jwst_gtvt --ra=16:52:58.9 --dec=02:24:03`

`$ jwst_gtvt --ra=253.2458 --dec=2.4008`

![Example Plot](docs/jwst_target_visibility.png "Example default plot output.")

For moving targets, use `jwst_mtvt`:

`$ jwst_mtvt Ceres`

You can achieve the same result above using the `--smallbody` flag and the HORIZONS Target designation ID:

`$ jwst_mtvt 1 --smallbody`

Periodic comets and most asteroids benefit from using the `--smallbody` flag.

`$ jwst_mtvt 2`  # Venus (no windows)

`$ jwst_mtvt 2 --smallbody`

![Example Plot](docs/mtvt_pallas.png "Example default plot output.")

Occasionally there will be too many matching designations entries for a single target

`$ jwst_mtvt 2P --smallbody`  # Comet Encke

    Record #  Epoch-yr  >MATCH DESIG<  Primary Desig  Name  
    --------  --------  -------------  -------------  -------------------------
    90000031    1786    2P             2P              Encke
    90000032    1796    2P             2P              Encke
    90000033    1805    2P             2P              Encke
    90000034    1819    2P             2P              Encke
    ........    ....    ..             ..              .....
    ........    ....    ..             ..              .....
    90000090    2016    2P             2P              Encke

`$ jwst_mtvt 90000090`  # Comet Encke

![Example Plot](docs/encke_visibility.png "Comet Encke.")

`$ jwst_mtvt 'C/2016 M1'`  # works with or without --smallbody

![Example Plot](docs/jwst_moving_target_visibility.png "A moving target example.")

You can specify the instrument via the `--instrument` flag.

`$ jwst_gtvt --ra=16:52:58.9 --dec=02:24:03 --instrument nircam`,

and the resulting plot will only contain the windows for the specified instrument.
The allowed values for `--instrument` are 'nircam', 'nirspec', 'niriss', 'miri', 'fgs', and 'v3pa' (case insensitive).

![Example Plot](docs/nircam_target_visibility.png "Example plot output when specifying instrument.")

You can save the text and figure ouput to a file instead of having it output to terminal with `--write_ephemeris` and `--write_plot`.

`$ jwst_gtvt --ra=16:52:58.9 --dec=02:24:03 --write_ephemeris visibility.csv --write_plot visibility.png`

Sharing output and reading data back in is easy:

    $ python
    >>> import pandas as pd
    >>> data = pd.read_csv('visibility.csv')
    >>> print(data[['MJD', ''NIRCAM_max_pa_angle']])
              MJD  NIRCAM_max_pa_angle
    0     59574.0                  NaN
    1     59575.0           232.359648
    2     59576.0           230.923010
    3     59577.0           229.540568
    4     59578.0           228.208743
    ...       ...                  ...
    1008  60582.0            79.433611
    1009  60583.0            78.278362
    1010  60584.0            77.103145
    1011  60585.0            75.906550
    1012  60586.0            74.687072

If you only want to plot a specific range of dates, rather than the entire available ephemeris you specify a `--start_date` or `--end_date` in ISO format (yyyy-mm-dd).
For example:

`$ jwst_gtvt --ra=16:52:58.9 --dec=02:24:03 --target_name "NGC 6240" --start_date 2023-01-01 --end_date 2024-01-01`

![Example Plot](docs/jwst_visibility_2023.png "Example plot output when specifying start and end dates.")

Below is an example of the full text output:

    $ jwst_gtvt --ra=16:52:58.9 --dec=02:24:03


    +------------------------------------------+
    | JWST General Target Visibility Tool      |
    +------------------------------------------+
    | Runtime/Date: YYYY-MM-DD   HR:MIN:SS     |
    +------------------------------------------+
    | Version Number: X.Y.Z                    |
    +------------------------------------------+
    
    RA: 253.24541666666664  Dec: 2.4008333333333334  Ecliptic Latitude: 24.284867
    -----------------------------------------------------------------------------
    
    Checked Interval [2021-12-26 00:00:00.000, 2024-10-03 00:00:00.000]
    
    +--------------------------------+--------------------------------+-------------------+------------------+----------------+
    | Window Start                   | Window End                     |   Window Duration |   V3 Angle Start |   V3 Angle End |
    |--------------------------------+--------------------------------+-------------------+------------------+----------------|
    | A.D. 2022-Feb-27 00:00:00.0000 | A.D. 2022-Apr-23 00:00:00.0000 |                55 |          279.221 |       250.7    |
    | A.D. 2022-Jul-14 00:00:00.0000 | A.D. 2022-Sep-11 00:00:00.0000 |                59 |          125.026 |        95.0131 |
    | A.D. 2023-Feb-27 00:00:00.0000 | A.D. 2023-Apr-24 00:00:00.0000 |                56 |          279.351 |       250.021  |
    | A.D. 2023-Jul-15 00:00:00.0000 | A.D. 2023-Sep-11 00:00:00.0000 |                58 |          124.475 |        95.0868 |
    | A.D. 2024-Feb-27 00:00:00.0000 | A.D. 2024-Apr-23 00:00:00.0000 |                56 |          279.472 |       250.166  |
    | A.D. 2024-Jul-14 00:00:00.0000 | A.D. 2024-Sep-10 00:00:00.0000 |                58 |          124.7   |        95.1624 |
    +--------------------------------+--------------------------------+-------------------+------------------+----------------+
    +--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------    +-------------   +-------------+
    |   NRC Max PA |   NRC Min PA |   NRS Max PA |   NRS Min PA |   NIS Max PA |   NIS Min PA |   MIR Max PA |   MIR Min PA |   FGS Max PA |   FGS Min PA |   V3 Max     PA    |   V3 Min PA |
    |--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------    +-------------   +-------------|
    |     283.067  |     275.15   |      61.6716 |      53.7553 |     283.749  |     275.833  |      288.013 |     280.097  |     281.928  |     274.012  |    283.    179     |    275.263  |
    |     283.222  |     274.159  |      61.8264 |      52.7638 |     283.904  |     274.841  |      288.168 |     279.106  |     282.083  |     273.021  |    283.    334     |    274.271  |
    |     283.378  |     273.168  |      61.9825 |      51.7728 |     284.06   |     273.85   |      288.324 |     278.115  |     282.239  |     272.03   |    283.    49      |    273.28   |
    |     282.958  |     272.754  |      61.5628 |      51.3589 |     283.64   |     273.436  |      287.905 |     277.701  |     281.82   |     271.616  |    283.    07      |    272.867  |
    |     282.54   |     272.339  |      61.1448 |      50.9441 |     283.222  |     273.022  |      287.487 |     277.286  |     281.402  |     271.201  |    282.    652     |    272.452  |
    |     282.123  |     271.923  |      60.7283 |      50.5282 |     282.806  |     272.606  |      287.07  |     276.87   |     280.985  |     270.785  |    282.    236     |    272.036  |
    |     281.708  |     271.506  |      60.3132 |      50.1111 |     282.391  |     272.189  |      286.655 |     276.453  |     280.57   |     270.368  |    281.    821     |    271.619  |
    |     281.294  |     271.087  |      59.8992 |      49.6924 |     281.977  |     271.77   |      286.241 |     276.034  |     280.156  |     269.949  |    281.    407     |    271.2    |
    |     280.881  |     270.667  |      59.486  |      49.2719 |     281.564  |     271.349  |      285.828 |     275.614  |     279.743  |     269.529  |    280.    994     |    270.78   |
    |     280.469  |     270.245  |      59.0735 |      48.8495 |     281.151  |     270.927  |      285.415 |     275.191  |     279.33   |     269.106  |    280.    581     |    270.357  |
    |     280.057  |     269.82   |      58.6614 |      48.4249 |     280.739  |     270.502  |      285.003 |     274.767  |     278.918  |     268.682  |    280.    169     |    269.933  |
    |     279.645  |     269.393  |      58.2496 |      47.9979 |     280.327  |     270.075  |      284.592 |     274.34   |     278.506  |     268.255  |    279.    757     |    269.506  |
    |     279.233  |     268.963  |      57.8378 |      47.5683 |     279.915  |     269.646  |      284.18  |     273.91   |     278.095  |     267.825  |    279.    345     |    269.076  |
    |     278.821  |     268.531  |      57.4258 |      47.1358 |     279.503  |     269.213  |      283.768 |     273.478  |     277.683  |     267.393  |    278.    933     |    268.643  |
    |     278.408  |     268.095  |      57.0134 |      46.7001 |     279.091  |     268.778  |      283.355 |     273.042  |     277.27   |     266.957  |    278.    521     |    268.208  |
    |     277.995  |     267.656  |      56.6003 |      46.2612 |     278.678  |     268.339  |      282.942 |     272.603  |     276.857  |     266.518  |    278.    108     |    267.769  |
    |     277.582  |     267.214  |      56.1864 |      45.8186 |     278.264  |     267.896  |      282.528 |     272.16   |     276.443  |     266.075  |    277.    694     |    267.326  |
    |     277.167  |     266.767  |      55.7715 |      45.3722 |     277.849  |     267.45   |      282.113 |     271.714  |     276.028  |     265.629  |    277.    279     |    266.88   |
    |     276.75   |     266.317  |      55.3552 |      44.9216 |     277.433  |     266.999  |      281.697 |     271.264  |     275.612  |     265.178  |    276.    863     |    266.429  |
    |     276.332  |     265.862  |      54.9374 |      44.4667 |     277.015  |     266.544  |      281.279 |     270.809  |     275.194  |     264.724  |    276.    445     |    265.974  |
    |     275.913  |     265.402  |      54.5178 |      44.0072 |     276.595  |     266.085  |      280.86  |     270.349  |     274.775  |     264.264  |    276.    025     |    265.515  |
    |     275.491  |     264.938  |      54.0962 |      43.5427 |     276.174  |     265.62   |      280.438 |     269.885  |     274.353  |     263.8    |    275.    604     |    265.05   |
    |     275.068  |     264.468  |      53.6724 |      43.0731 |     275.75   |     265.151  |      280.014 |     269.415  |     273.929  |     263.33   |    275.    18      |    264.581  |
    |     274.641  |     263.993  |      53.2461 |      42.5979 |     275.324  |     264.675  |      279.588 |     268.94   |     273.503  |     262.855  |    274.    754     |    264.106  |
    |     274.212  |     263.512  |      52.8171 |      42.1169 |     274.895  |     264.194  |      279.159 |     268.459  |     273.074  |     262.374  |    274.    325     |    263.625  |
    |     273.78   |     263.025  |      52.385  |      41.6299 |     274.463  |     263.707  |      278.727 |     267.972  |     272.642  |     261.887  |    273.    893     |    263.138  |
    |     273.345  |     262.531  |      51.9497 |      41.1364 |     274.027  |     263.214  |      278.292 |     267.478  |     272.207  |     261.393  |    273.    457     |    262.644  |
    |     272.906  |     262.031  |      51.5109 |      40.6361 |     273.588  |     262.714  |      277.853 |     266.978  |     271.768  |     260.893  |    273.    019     |    262.144  |
    |     272.463  |     261.524  |      51.0683 |      40.1287 |     273.146  |     262.206  |      277.41  |     266.471  |     271.325  |     260.386  |    272.    576     |    261.636  |
    |     272.017  |     261.009  |      50.6215 |      39.6139 |     272.699  |     261.691  |      276.963 |     265.956  |     270.878  |     259.871  |    272.    129     |    261.122  |
    |     271.565  |     260.486  |      50.1704 |      39.0912 |     272.248  |     261.169  |      276.512 |     265.433  |     270.427  |     259.348  |    271.    678     |    260.599  |
    |     271.11   |     259.955  |      49.7145 |      38.5604 |     271.792  |     260.638  |      276.056 |     264.902  |     269.971  |     258.817  |    271.    222     |    260.068  |
    |     270.649  |     259.416  |      49.2537 |      38.0209 |     271.331  |     260.098  |      275.596 |     264.363  |     269.51   |     258.278  |    270.    761     |    259.529  |
    |     270.183  |     258.868  |      48.7875 |      37.4724 |     270.865  |     259.55   |      275.129 |     263.814  |     269.044  |     257.729  |    270.    295     |    258.98   |
    |     269.711  |     258.31   |      48.3156 |      36.9145 |     270.393  |     258.992  |      274.657 |     263.256  |     268.572  |     257.171  |    269.    823     |    258.422  |
    |     269.233  |     257.742  |      47.8377 |      36.3468 |     269.915  |     258.424  |      274.18  |     262.689  |     268.095  |     256.604  |    269.    345     |    257.854  |
    |     268.748  |     257.164  |      47.3534 |      35.7687 |     269.431  |     257.846  |      273.695 |     262.111  |     267.61   |     256.026  |    268.    861     |    257.276  |
    |     268.257  |     256.575  |      46.8623 |      35.1798 |     268.94   |     257.257  |      273.204 |     261.522  |     267.119  |     255.437  |    268.    37      |    256.687  |
    |     267.759  |     255.975  |      46.3641 |      34.5796 |     268.442  |     256.657  |      272.706 |     260.922  |     266.621  |     254.836  |    267.    872     |    256.087  |
    |     267.253  |     255.363  |      45.8583 |      33.9676 |     267.936  |     256.045  |      272.2   |     260.31   |     266.115  |     254.224  |    267.    366     |    255.475  |
    |     266.74   |     254.738  |      45.3446 |      33.3433 |     267.422  |     255.421  |      271.686 |     259.685  |     265.601  |     253.6    |    266.    852     |    254.851  |
    |     266.218  |     254.101  |      44.8224 |      32.7061 |     266.9    |     254.784  |      271.164 |     259.048  |     265.079  |     252.963  |    266.    33      |    254.214  |
    |     265.686  |     253.451  |      44.2913 |      32.0554 |     266.369  |     254.133  |      270.633 |     258.397  |     264.548  |     252.312  |    265.    799     |    253.563  |
    |     265.146  |     252.786  |      43.7509 |      31.3907 |     265.828  |     253.468  |      270.093 |     257.733  |     264.008  |     251.648  |    265.    259     |    252.898  |
    |     264.596  |     252.106  |      43.2006 |      30.7113 |     265.278  |     252.789  |      269.543 |     257.053  |     263.457  |     250.968  |    264.    708     |    252.219  |
    |     264.035  |     251.412  |      42.64   |      30.0165 |     264.717  |     252.094  |      268.982 |     256.358  |     262.897  |     250.273  |    264.    148     |    251.524  |
    |     263.463  |     250.701  |      42.0683 |      29.3056 |     264.146  |     251.383  |      268.41  |     255.648  |     262.325  |     249.562  |    263.    576     |    250.813  |
    |     262.88   |     249.973  |      41.4852 |      28.5781 |     263.563  |     250.656  |      267.827 |     254.92   |     261.742  |     248.835  |    262.    993     |    250.086  |
    |     262.285  |     249.228  |      40.89   |      27.833  |     262.967  |     249.911  |      267.232 |     254.175  |     261.147  |     248.09   |    262.    398     |    249.341  |
    |     261.677  |     248.465  |      40.282  |      27.0696 |     262.359  |     249.147  |      266.624 |     253.412  |     260.539  |     247.326  |    261.    79      |    248.577  |
    |     261.056  |     247.682  |      39.6606 |      26.2872 |     261.738  |     248.365  |      266.002 |     252.629  |     259.917  |     246.544  |    261.    168     |    247.795  |
    |     260.42   |     246.88   |      39.025  |      25.4848 |     261.103  |     247.562  |      265.367 |     251.827  |     259.282  |     245.742  |    260.    533     |    246.992  |
    |     259.77   |     246.057  |      38.3746 |      24.6616 |     260.452  |     246.739  |      264.717 |     251.004  |     258.631  |     244.918  |    259.    882     |    246.169  |
    |     259.104  |     245.212  |      37.7086 |      23.8167 |     259.786  |     245.894  |      264.051 |     250.159  |     257.965  |     244.074  |    259.    216     |    245.324  |
    |     258.421  |     244.344  |      37.0262 |      22.949  |     259.104  |     245.027  |      263.368 |     249.291  |     257.283  |     243.206  |    258.    534     |    244.457  |
    |     257.722  |     243.453  |      36.3265 |      22.0577 |     258.404  |     244.135  |      262.668 |     248.4    |     256.583  |     242.315  |    257.    834     |    243.565  |
    |     132.134  |     117.693  |     270.738  |     256.298  |     132.816  |     118.376  |      137.08  |     122.64   |     130.995  |     116.555  |    132.    246     |    117.806  |
    |     131.253  |     117.002  |     269.858  |     255.607  |     131.936  |     117.685  |      136.2   |     121.949  |     130.115  |     115.864  |    131.    366     |    117.115  |
    |     130.396  |     116.328  |     269.001  |     254.933  |     131.079  |     117.01   |      135.343 |     121.275  |     129.258  |     115.19   |    130.    509     |    116.44   |
    |     129.561  |     115.67   |     268.166  |     254.275  |     130.243  |     116.352  |      134.508 |     120.616  |     128.423  |     114.531  |    129.    674     |    115.782  |
    |     128.747  |     115.027  |     267.352  |     253.631  |     129.429  |     115.709  |      133.694 |     119.973  |     127.609  |     113.888  |    128.    859     |    115.139  |
    |     127.953  |     114.398  |     266.558  |     253.003  |     128.635  |     115.08   |      132.9   |     119.345  |     126.815  |     113.26   |    128.    066     |    114.511  |
    |     127.178  |     113.783  |     265.783  |     252.388  |     127.861  |     114.466  |      132.125 |     118.73   |     126.04   |     112.645  |    127.    291     |    113.896  |
    |     126.422  |     113.181  |     265.027  |     251.786  |     127.105  |     113.864  |      131.369 |     118.128  |     125.284  |     112.043  |    126.    535     |    113.294  |
    |     125.684  |     112.592  |     264.288  |     251.197  |     126.366  |     113.275  |      130.63  |     117.539  |     124.545  |     111.454  |    125.    796     |    112.705  |
    |     124.962  |     112.015  |     263.567  |     250.619  |     125.644  |     112.697  |      129.909 |     116.961  |     123.824  |     110.876  |    125.    075     |    112.127  |
    |     124.257  |     111.448  |     262.861  |     250.053  |     124.939  |     112.131  |      129.203 |     116.395  |     123.118  |     110.31   |    124.    369     |    111.561  |
    |     123.567  |     110.893  |     262.172  |     249.497  |     124.249  |     111.575  |      128.513 |     115.839  |     122.428  |     109.754  |    123.    679     |    111.005  |
    |     122.892  |     110.347  |     261.496  |     248.952  |     123.574  |     111.029  |      127.838 |     115.294  |     121.753  |     109.209  |    123.    004     |    110.46   |
    |     122.231  |     109.811  |     260.836  |     248.416  |     122.913  |     110.494  |      127.178 |     114.758  |     121.092  |     108.673  |    122.    343     |    109.924  |
    |     121.583  |     109.284  |     260.188  |     247.889  |     122.266  |     109.967  |      126.53  |     114.231  |     120.445  |     108.146  |    121.    696     |    109.397  |
    |     120.949  |     108.766  |     259.554  |     247.371  |     121.632  |     109.449  |      125.896 |     113.713  |     119.811  |     107.628  |    121.    062     |    108.879  |
    |     120.327  |     108.256  |     258.932  |     246.861  |     121.01   |     108.939  |      125.274 |     113.203  |     119.189  |     107.118  |    120.    44      |    108.369  |
    |     119.717  |     107.754  |     258.322  |     246.359  |     120.4    |     108.436  |      124.664 |     112.701  |     118.579  |     106.616  |    119.    83      |    107.866  |
    |     119.119  |     107.259  |     257.724  |     245.864  |     119.801  |     107.942  |      124.066 |     112.206  |     117.981  |     106.121  |    119.    231     |    107.372  |
    |     118.531  |     106.771  |     257.136  |     245.376  |     119.214  |     107.454  |      123.478 |     111.718  |     117.393  |     105.633  |    118.    644     |    106.884  |
    |     117.954  |     106.29   |     256.559  |     244.895  |     118.636  |     106.972  |      122.901 |     111.237  |     116.816  |     105.152  |    118.    066     |    106.403  |
    |     117.387  |     105.815  |     255.992  |     244.42   |     118.069  |     106.498  |      122.333 |     110.762  |     116.248  |     104.677  |    117.    499     |    105.928  |
    |     116.829  |     105.346  |     255.434  |     243.951  |     117.511  |     106.028  |      121.776 |     110.293  |     115.691  |     104.208  |    116.    941     |    105.459  |
    |     116.28   |     104.883  |     254.885  |     243.487  |     116.963  |     105.565  |      121.227 |     109.829  |     115.142  |     103.744  |    116.    393     |    104.995  |
    |     115.74   |     104.424  |     254.345  |     243.029  |     116.423  |     105.107  |      120.687 |     109.371  |     114.602  |     103.286  |    115.    853     |    104.537  |
    |     115.209  |     103.971  |     253.814  |     242.576  |     115.891  |     104.654  |      120.156 |     108.918  |     114.071  |     102.833  |    115.    321     |    104.084  |
    |     114.685  |     103.522  |     253.29   |     242.127  |     115.368  |     104.205  |      119.632 |     108.469  |     113.547  |     102.384  |    114.    798     |    103.635  |
    |     114.169  |     103.078  |     252.774  |     241.683  |     114.852  |     103.761  |      119.116 |     108.025  |     113.031  |     101.94   |    114.    282     |    103.191  |
    |     113.661  |     102.638  |     252.266  |     241.243  |     114.343  |     103.32   |      118.607 |     107.585  |     112.522  |     101.5    |    113.    773     |    102.751  |
    |     113.159  |     102.202  |     251.764  |     240.807  |     113.841  |     102.884  |      118.106 |     107.149  |     112.021  |     101.063  |    113.    271     |    102.314  |
    |     112.664  |     101.769  |     251.269  |     240.374  |     113.346  |     102.451  |      117.61  |     106.716  |     111.525  |     100.631  |    112.    776     |    101.882  |
    |     112.175  |     101.34   |     250.78   |     239.944  |     112.857  |     102.022  |      117.122 |     106.286  |     111.036  |     100.201  |    112.    287     |    101.452  |
    |     111.692  |     100.913  |     250.297  |     239.518  |     112.374  |     101.596  |      116.639 |     105.86   |     110.554  |      99.7749 |    111.    804     |    101.026  |
    |     111.215  |     100.49   |     249.819  |     239.094  |     111.897  |     101.172  |      116.161 |     105.436  |     110.076  |      99.3513 |    111.    327     |    100.602  |
    |     110.743  |     100.069  |     249.348  |     238.673  |     111.425  |     100.751  |      115.69  |     105.015  |     109.604  |      98.9303 |    110.    855     |    100.181  |
    |     110.276  |      99.65   |     248.881  |     238.255  |     110.959  |     100.332  |      115.223 |     104.597  |     109.138  |      98.5117 |    110.    389     |     99.7625 |
    |     109.814  |      99.2335 |     248.419  |     237.838  |     110.497  |      99.9159 |      114.761 |     104.18   |     108.676  |      98.0953 |    109.    927     |     99.3461 |
    |     109.357  |      98.819  |     247.962  |     237.424  |     110.04   |      99.5014 |      114.304 |     103.766  |     108.219  |      97.6807 |    109.    47      |     98.9315 |
    |     108.904  |      98.4062 |     247.509  |     237.011  |     109.587  |      99.0886 |      113.851 |     103.353  |     107.766  |      97.2679 |    109.    017     |     98.5187 |
    |     108.456  |      97.9949 |     247.061  |     236.6    |     109.138  |      98.6773 |      113.403 |     102.942  |     107.317  |      96.8566 |    108.    568     |     98.1074 |
    |     108.011  |      97.5849 |     246.616  |     236.19   |     108.693  |      98.2673 |      112.958 |     102.532  |     106.873  |      96.4466 |    108.    124     |     97.6974 |
    |     107.57   |      97.176  |     246.175  |     235.781  |     108.252  |      97.8584 |      112.517 |     102.123  |     106.432  |      96.0377 |    107.    682     |     97.2885 |
    |     107.132  |      96.768  |     245.737  |     235.373  |     107.815  |      97.4504 |      112.079 |     101.715  |     105.994  |      95.6297 |    107.    245     |     96.8805 |
    |     106.698  |      96.3607 |     245.303  |     234.966  |     107.38   |      97.0431 |      111.645 |     101.308  |     105.56   |      95.2224 |    106.    81      |     96.4733 |
    |     106.266  |      95.9539 |     244.871  |     234.559  |     106.949  |      96.6364 |      111.213 |     100.901  |     105.128  |      94.8157 |    106.    379     |     96.0665 |
    |     105.838  |      95.5475 |     244.443  |     234.152  |     106.52   |      96.2299 |      110.785 |     100.494  |     104.7    |      94.4092 |    105.    95      |     95.66   |
    |     105.412  |      95.1411 |     244.017  |     233.746  |     106.094  |      95.8236 |      110.359 |     100.088  |     104.273  |      94.0029 |    105.    524     |     95.2537 |
    |     104.988  |      94.7347 |     243.593  |     233.34   |     105.67   |      95.4172 |      109.935 |      99.6815 |     103.85   |      93.5965 |    105.    101     |     94.8473 |
    |     104.567  |      94.3281 |     243.171  |     232.933  |     105.249  |      95.0105 |      109.513 |      99.2749 |     103.428  |      93.1898 |    104.    679     |     94.4406 |
    |     104.147  |      93.921  |     242.752  |     232.526  |     104.829  |      94.6034 |      109.094 |      98.8678 |     103.009  |      92.7827 |    104.    26      |     94.0335 |
    |     103.729  |      93.5132 |     242.334  |     232.118  |     104.412  |      94.1957 |      108.676 |      98.46   |     102.591  |      92.375  |    103.    842     |     93.6258 |
    |     103.313  |      93.1047 |     241.918  |     231.71   |     103.996  |      93.7871 |      108.26  |      98.0515 |     102.175  |      91.9664 |    103.    426     |     93.2172 |
    |     102.898  |      92.6951 |     241.503  |     231.3    |     103.581  |      93.3775 |      107.845 |      97.6419 |     101.76   |      91.5568 |    103.    011     |     92.8076 |
    |     102.485  |      92.2843 |     241.09   |     230.889  |     103.167  |      92.9667 |      107.432 |      97.2311 |     101.347  |      91.146  |    102.    597     |     92.3968 |
    |     102.072  |      91.8721 |     240.677  |     230.477  |     102.755  |      92.5545 |      107.019 |      96.8189 |     100.934  |      90.7338 |    102.    185     |     91.9846 |
    |     101.66   |      91.4583 |     240.265  |     230.063  |     102.343  |      92.1407 |      106.607 |      96.4051 |     100.522  |      90.32   |    101.    773     |     91.5708 |
    |     101.249  |      91.0427 |     239.854  |     229.648  |     101.932  |      91.7251 |      106.196 |      95.9895 |     100.111  |      89.9044 |    101.    362     |     91.1552 |
    |     100.544  |      90.9203 |     239.148  |     229.525  |     101.226  |      91.6027 |      105.49  |      95.8671 |      99.4053 |      89.782  |    100.    656     |     91.0329 |
    |      99.5752 |      91.0585 |     238.18   |     229.663  |     100.258  |      91.7409 |      104.522 |      96.0053 |      98.4369 |      89.9202 |     99.    6877    |     91.171  |
    |      98.6058 |      91.1954 |     237.211  |     229.8    |      99.2882 |      91.8778 |      103.553 |      96.1422 |      97.4675 |      90.0571 |     98.    7183    |     91.3079 |
    |     283.068  |     275.408  |      61.6728 |      54.0134 |     283.75   |     276.091  |      288.015 |     280.355  |     281.93   |     274.27   |    283.    18      |    275.521  |
    |     283.222  |     274.414  |      61.8274 |      53.0194 |     283.905  |     275.097  |      288.169 |     279.361  |     282.084  |     273.276  |    283.    335     |    274.527  |
    |     283.379  |     273.421  |      61.9834 |      52.0256 |     284.061  |     274.103  |      288.325 |     278.368  |     282.24   |     272.282  |    283.    491     |    273.533  |
    |     283.084  |     272.879  |      61.6888 |      51.4839 |     283.766  |     273.561  |      288.031 |     277.826  |     281.946  |     271.741  |    283.    196     |    272.992  |
    |     282.664  |     272.463  |      61.2692 |      51.0681 |     283.347  |     273.146  |      287.611 |     277.41   |     281.526  |     271.325  |    282.    777     |    272.576  |
    |     282.246  |     272.046  |      60.8512 |      50.6511 |     282.929  |     272.729  |      287.193 |     276.993  |     281.108  |     270.908  |    282.    359     |    272.159  |
    |     281.83   |     271.628  |      60.4344 |      50.2329 |     282.512  |     272.31   |      286.776 |     276.575  |     280.691  |     270.49   |    281.    942     |    271.741  |
    |     281.414  |     271.208  |      60.0188 |      49.8132 |     282.096  |     271.891  |      286.361 |     276.155  |     280.276  |     270.07   |    281.    526     |    271.321  |
    |     280.999  |     270.787  |      59.6041 |      49.3918 |     281.682  |     271.469  |      285.946 |     275.734  |     279.861  |     269.649  |    281.    112     |    270.899  |
    |     280.585  |     270.363  |      59.1901 |      48.9684 |     281.268  |     271.046  |      285.532 |     275.31   |     279.447  |     269.225  |    280.    698     |    270.476  |
    |     280.172  |     269.938  |      58.7765 |      48.5428 |     280.854  |     270.62   |      285.118 |     274.885  |     279.033  |     268.8    |    280.    284     |    270.05   |
    |     279.758  |     269.51   |      58.3632 |      48.1148 |     280.441  |     270.192  |      284.705 |     274.457  |     278.62   |     268.372  |    279.    871     |    269.622  |
    |     279.345  |     269.079  |      57.9499 |      47.6842 |     280.027  |     269.762  |      284.292 |     274.026  |     278.207  |     267.941  |    279.    458     |    269.192  |
    |     278.932  |     268.646  |      57.5365 |      47.2508 |     279.614  |     269.328  |      283.878 |     273.593  |     277.793  |     267.508  |    279.    044     |    268.758  |
    |     278.518  |     268.209  |      57.1227 |      46.8142 |     279.2    |     268.892  |      283.465 |     273.156  |     277.38   |     267.071  |    278.    63      |    268.322  |
    |     278.103  |     267.769  |      56.7082 |      46.3743 |     278.786  |     268.452  |      283.05  |     272.716  |     276.965  |     266.631  |    278.    216     |    267.882  |
    |     277.688  |     267.326  |      56.2929 |      45.9309 |     278.37   |     268.008  |      282.635 |     272.273  |     276.55   |     266.188  |    277.    801     |    267.439  |
    |     277.272  |     266.879  |      55.8766 |      45.4836 |     277.954  |     267.561  |      282.219 |     271.825  |     276.133  |     265.74   |    277.    384     |    266.991  |
    |     276.854  |     266.427  |      55.459  |      45.0322 |     277.537  |     267.11   |      281.801 |     271.374  |     275.716  |     265.289  |    276.    967     |    266.54   |
    |     276.435  |     265.972  |      55.0398 |      44.5764 |     277.117  |     266.654  |      281.382 |     270.918  |     275.297  |     264.833  |    276.    548     |    266.084  |
    |     276.014  |     265.511  |      54.619  |      44.1161 |     276.697  |     266.194  |      280.961 |     270.458  |     274.876  |     264.373  |    276.    127     |    265.624  |
    |     275.591  |     265.046  |      54.1961 |      43.6509 |     276.274  |     265.728  |      280.538 |     269.993  |     274.453  |     263.908  |    275.    704     |    265.159  |
    |     275.166  |     264.576  |      53.7711 |      43.1805 |     275.849  |     265.258  |      280.113 |     269.522  |     274.028  |     263.437  |    275.    279     |    264.688  |
    |     274.739  |     264.1    |      53.3436 |      42.7046 |     275.421  |     264.782  |      279.685 |     269.047  |     273.6    |     262.961  |    274.    851     |    264.212  |
    |     274.308  |     263.618  |      52.9134 |      42.223  |     274.991  |     264.3    |      279.255 |     268.565  |     273.17   |     262.48   |    274.    421     |    263.731  |
    |     273.875  |     263.13   |      52.4802 |      41.7353 |     274.558  |     263.813  |      278.822 |     268.077  |     272.737  |     261.992  |    273.    988     |    263.243  |
    |     273.439  |     262.636  |      52.0438 |      41.2412 |     274.121  |     263.319  |      278.386 |     267.583  |     272.301  |     261.498  |    273.    551     |    262.749  |
    |     272.999  |     262.136  |      51.6039 |      40.7404 |     273.681  |     262.818  |      277.946 |     267.082  |     271.861  |     260.997  |    273.    112     |    262.248  |
    |     272.555  |     261.628  |      51.1602 |      40.2326 |     273.238  |     262.31   |      277.502 |     266.575  |     271.417  |     260.489  |    272.    668     |    261.74   |
    |     272.108  |     261.113  |      50.7125 |      39.7174 |     272.79   |     261.795  |      277.054 |     266.059  |     270.969  |     259.974  |    272.    22      |    261.225  |
    |     271.656  |     260.59   |      50.2605 |      39.1944 |     272.338  |     261.272  |      276.602 |     265.536  |     270.517  |     259.451  |    271.    768     |    260.702  |
    |     271.199  |     260.058  |      49.8038 |      38.6634 |     271.881  |     260.741  |      276.146 |     265.005  |     270.061  |     258.92   |    271.    311     |    260.171  |
    |     270.737  |     259.519  |      49.3422 |      38.1238 |     271.42   |     260.201  |      275.684 |     264.466  |     269.599  |     258.381  |    270.    85      |    259.631  |
    |     270.27   |     258.97   |      48.8753 |      37.5753 |     270.953  |     259.653  |      275.217 |     263.917  |     269.132  |     257.832  |    270.    383     |    259.083  |
    |     269.798  |     258.413  |      48.4028 |      37.0176 |     270.48   |     259.095  |      274.745 |     263.359  |     268.66   |     257.274  |    269.    91      |    258.525  |
    |     269.32   |     257.845  |      47.9244 |      36.45   |     270.002  |     258.528  |      274.266 |     262.792  |     268.181  |     256.707  |    269.    432     |    257.958  |
    |     268.835  |     257.267  |      47.4398 |      35.8724 |     269.517  |     257.95   |      273.782 |     262.214  |     267.697  |     256.129  |    268.    947     |    257.38   |
    |     268.344  |     256.679  |      46.9484 |      35.284  |     269.026  |     257.362  |      273.29  |     261.626  |     267.205  |     255.541  |    268.    456     |    256.792  |
    |     267.845  |     256.08   |      46.4501 |      34.6845 |     268.528  |     256.762  |      272.792 |     261.026  |     266.707  |     254.941  |    267.    958     |    256.192  |
    |     267.339  |     255.469  |      45.9443 |      34.0735 |     268.022  |     256.151  |      272.286 |     260.415  |     266.201  |     254.33   |    267.    452     |    255.581  |
    |     266.826  |     254.845  |      45.4307 |      33.4502 |     267.508  |     255.528  |      271.773 |     259.792  |     265.688  |     253.707  |    266.    938     |    254.958  |
    |     266.304  |     254.209  |      44.9088 |      32.8143 |     266.986  |     254.892  |      271.251 |     259.156  |     265.166  |     253.071  |    266.    416     |    254.322  |
    |     265.773  |     253.56   |      44.3782 |      32.1651 |     266.456  |     254.243  |      270.72  |     258.507  |     264.635  |     252.422  |    265.    886     |    253.673  |
    |     265.234  |     252.897  |      43.8384 |      31.5021 |     265.916  |     253.58   |      270.18  |     257.844  |     264.095  |     251.759  |    265.    346     |    253.01   |
    |     264.684  |     252.22   |      43.2889 |      30.8246 |     265.366  |     252.902  |      269.631 |     257.166  |     263.546  |     251.081  |    264.    797     |    252.332  |
    |     264.124  |     251.527  |      42.7293 |      30.1319 |     264.807  |     252.209  |      269.071 |     256.474  |     262.986  |     250.389  |    264.    237     |    251.64   |
    |     263.554  |     250.819  |      42.1588 |      29.4236 |     264.236  |     251.501  |      268.501 |     255.765  |     262.416  |     249.68   |    263.    667     |    250.931  |
    |     262.972  |     250.094  |      41.5771 |      28.6987 |     263.655  |     250.776  |      267.919 |     255.041  |     261.834  |     248.956  |    263.    085     |    250.206  |
    |     262.379  |     249.352  |      40.9835 |      27.9566 |     263.061  |     250.034  |      267.325 |     254.299  |     261.24   |     248.213  |    262.    491     |    249.464  |
    |     261.773  |     248.592  |      40.3774 |      27.1966 |     262.455  |     249.274  |      266.719 |     253.539  |     260.634  |     247.453  |    261.    885     |    248.704  |
    |     261.153  |     247.813  |      39.7581 |      26.4178 |     261.836  |     248.495  |      266.1   |     252.76   |     260.015  |     246.675  |    261.    266     |    247.925  |
    |     260.52   |     247.014  |      39.125  |      25.6194 |     261.203  |     247.697  |      265.467 |     251.961  |     259.382  |     245.876  |    260.    633     |    247.127  |
    |     259.872  |     246.196  |      38.4773 |      24.8005 |     260.555  |     246.878  |      264.819 |     251.142  |     258.734  |     245.057  |    259.    985     |    246.308  |
    |     259.209  |     245.355  |      37.8143 |      23.9603 |     259.892  |     246.038  |      264.156 |     250.302  |     258.071  |     244.217  |    259.    322     |    245.468  |
    |     258.53   |     244.493  |      37.1351 |      23.0977 |     259.213  |     245.175  |      263.477 |     249.44   |     257.392  |     243.355  |    258.    643     |    244.605  |
    |     257.834  |     243.607  |      36.439  |      22.2118 |     258.517  |     244.289  |      262.781 |     248.554  |     256.696  |     242.469  |    257.    947     |    243.72   |
    |     257.12   |     242.697  |      35.7251 |      21.3017 |     257.803  |     243.379  |      262.067 |     247.644  |     255.982  |     241.559  |    257.    233     |    242.809  |
    |     131.519  |     117.206  |     270.124  |     255.811  |     132.202  |     117.888  |      136.466 |     122.153  |     130.381  |     116.068  |    131.    632     |    117.319  |
    |     130.654  |     116.525  |     269.259  |     255.13   |     131.336  |     117.208  |      135.601 |     121.472  |     129.516  |     115.387  |    130.    766     |    116.638  |
    |     129.811  |     115.861  |     268.416  |     254.466  |     130.493  |     116.544  |      134.757 |     120.808  |     128.672  |     114.723  |    129.    923     |    115.974  |
    |     128.989  |     115.212  |     267.594  |     253.817  |     129.671  |     115.895  |      133.936 |     120.159  |     127.851  |     114.074  |    129.    101     |    115.325  |
    |     128.187  |     114.578  |     266.792  |     253.183  |     128.87   |     115.261  |      133.134 |     119.525  |     127.049  |     113.44   |    128.    3       |    114.691  |
    |     127.405  |     113.958  |     266.01   |     252.563  |     128.088  |     114.641  |      132.352 |     118.905  |     126.267  |     112.82   |    127.    518     |    114.071  |
    |     126.642  |     113.351  |     265.247  |     251.956  |     127.325  |     114.034  |      131.589 |     118.298  |     125.504  |     112.213  |    126.    755     |    113.464  |
    |     125.897  |     112.757  |     264.502  |     251.362  |     126.58   |     113.44   |      130.844 |     117.704  |     124.759  |     111.619  |    126.    01      |    112.87   |
    |     125.169  |     112.175  |     263.774  |     250.78   |     125.851  |     112.857  |      130.116 |     117.122  |     124.031  |     111.037  |    125.    282     |    112.287  |
    |     124.457  |     111.604  |     263.062  |     250.209  |     125.14   |     112.287  |      129.404 |     116.551  |     123.319  |     110.466  |    124.    57      |    111.717  |
    |     123.762  |     111.044  |     262.366  |     249.649  |     124.444  |     111.727  |      128.708 |     115.991  |     122.623  |     109.906  |    123.    874     |    111.157  |
    |     123.081  |     110.494  |     261.686  |     249.099  |     123.763  |     111.177  |      128.027 |     115.441  |     121.942  |     109.356  |    123.    193     |    110.607  |
    |     122.414  |     109.955  |     261.019  |     248.559  |     123.097  |     110.637  |      127.361 |     114.901  |     121.276  |     108.816  |    122.    527     |    110.067  |
    |     121.762  |     109.424  |     260.367  |     248.029  |     122.444  |     110.106  |      126.708 |     114.371  |     120.623  |     108.286  |    121.    874     |    109.537  |
    |     121.122  |     108.902  |     259.727  |     247.507  |     121.805  |     109.585  |      126.069 |     113.849  |     119.984  |     107.764  |    121.    235     |    109.015  |
    |     120.495  |     108.389  |     259.1    |     246.994  |     121.178  |     109.071  |      125.442 |     113.336  |     119.357  |     107.25   |    120.    608     |    108.501  |
    |     119.881  |     107.883  |     258.486  |     246.488  |     120.563  |     108.566  |      124.827 |     112.83   |     118.742  |     106.745  |    119.    993     |    107.996  |
    |     119.277  |     107.385  |     257.882  |     245.99   |     119.96   |     108.068  |      124.224 |     112.332  |     118.139  |     106.247  |    119.    39      |    107.498  |
    |     118.685  |     106.894  |     257.29   |     245.499  |     119.368  |     107.577  |      123.632 |     111.841  |     117.547  |     105.756  |    118.    798     |    107.007  |
    |     118.104  |     106.41   |     256.709  |     245.015  |     118.786  |     107.093  |      123.051 |     111.357  |     116.966  |     105.272  |    118.    216     |    106.523  |
    |     117.533  |     105.932  |     256.137  |     244.537  |     118.215  |     106.615  |      122.479 |     110.879  |     116.394  |     104.794  |    117.    645     |    106.045  |
    |     116.971  |     105.461  |     255.576  |     244.066  |     117.653  |     106.143  |      121.918 |     110.408  |     115.833  |     104.322  |    117.    083     |    105.573  |
    |     116.418  |     104.995  |     255.023  |     243.6    |     117.101  |     105.677  |      121.365 |     109.941  |     115.28   |     103.856  |    116.    531     |    105.107  |
    |     115.875  |     104.534  |     254.48   |     243.139  |     116.557  |     105.216  |      120.822 |     109.481  |     114.737  |     103.396  |    115.    987     |    104.647  |
    |     115.34   |     104.078  |     253.945  |     242.683  |     116.022  |     104.761  |      120.287 |     109.025  |     114.202  |     102.94   |    115.    452     |    104.191  |
    |     114.813  |     103.628  |     253.418  |     242.232  |     115.495  |     104.31   |      119.76  |     108.574  |     113.675  |     102.489  |    114.    925     |    103.74   |
    |     114.294  |     103.181  |     252.899  |     241.786  |     114.976  |     103.864  |      119.24  |     108.128  |     113.155  |     102.043  |    114.    406     |    103.294  |
    |     113.782  |     102.739  |     252.387  |     241.344  |     114.464  |     103.421  |      118.729 |     107.686  |     112.644  |     101.601  |    113.    894     |    102.851  |
    |     113.277  |     102.301  |     251.882  |     240.905  |     113.959  |     102.983  |      118.224 |     107.247  |     112.139  |     101.162  |    113.    39      |    102.413  |
    |     112.779  |     101.866  |     251.384  |     240.471  |     113.461  |     102.548  |      117.726 |     106.813  |     111.641  |     100.728  |    112.    891     |    101.979  |
    |     112.287  |     101.435  |     250.892  |     240.04   |     112.97   |     102.117  |      117.234 |     106.382  |     111.149  |     100.296  |    112.    4       |    101.547  |
    |     111.802  |     101.007  |     250.406  |     239.612  |     112.484  |     101.689  |      116.748 |     105.953  |     110.663  |      99.8684 |    111.    914     |    101.119  |
    |     111.322  |     100.581  |     249.927  |     239.186  |     112.004  |     101.264  |      116.269 |     105.528  |     110.184  |      99.4432 |    111.    434     |    100.694  |
    |     110.848  |     100.159  |     249.452  |     238.764  |     111.53   |     100.841  |      115.794 |     105.106  |     109.709  |      99.0207 |    110.    96      |    100.271  |
    |     110.378  |      99.7389 |     248.983  |     238.344  |     111.061  |     100.421  |      115.325 |     104.686  |     109.24   |      98.6006 |    110.    491     |     99.8514 |
    |     109.914  |      99.321  |     248.519  |     237.926  |     110.597  |     100.003  |      114.861 |     104.268  |     108.776  |      98.1828 |    110.    027     |     99.4336 |
    |     109.455  |      98.9052 |     248.06   |     237.51   |     110.138  |      99.5876 |      114.402 |     103.852  |     108.317  |      97.7669 |    109.    568     |     99.0178 |
    |     109      |      98.4912 |     247.605  |     237.096  |     109.683  |      99.1736 |      113.947 |     103.438  |     107.862  |      97.3529 |    109.    113     |     98.6037 |
    |     108.55   |      98.0787 |     247.155  |     236.684  |     109.232  |      98.7611 |      113.496 |     103.026  |     107.411  |      96.9404 |    108.    662     |     98.1913 |
    |     108.103  |      97.6676 |     246.708  |     236.273  |     108.785  |      98.35   |      113.05  |     102.614  |     106.965  |      96.5294 |    108.    216     |     97.7802 |
    |     107.66   |      97.2577 |     246.265  |     235.863  |     108.343  |      97.9401 |      112.607 |     102.205  |     106.522  |      96.1194 |    107.    773     |     97.3703 |
    |     107.221  |      96.8488 |     245.826  |     235.454  |     107.903  |      97.5312 |      112.168 |     101.796  |     106.083  |      95.7105 |    107.    333     |     96.9613 |
    |     106.785  |      96.4406 |     245.39   |     235.045  |     107.467  |      97.123  |      111.732 |     101.387  |     105.647  |      95.3023 |    106.    897     |     96.5532 |
    |     106.352  |      96.033  |     244.957  |     234.638  |     107.034  |      96.7154 |      111.299 |     100.98   |     105.214  |      94.8947 |    106.    464     |     96.1456 |
    |     105.922  |      95.6258 |     244.527  |     234.231  |     106.604  |      96.3082 |      110.869 |     100.573  |     104.783  |      94.4875 |    106.    034     |     95.7384 |
    |     105.494  |      95.2188 |     244.099  |     233.824  |     106.177  |      95.9012 |      110.441 |     100.166  |     104.356  |      94.0805 |    105.    607     |     95.3313 |
    |     105.069  |      94.8118 |     243.674  |     233.417  |     105.752  |      95.4942 |      110.016 |      99.7586 |     103.931  |      93.6735 |    105.    182     |     94.9243 |
    |     104.647  |      94.4046 |     243.252  |     233.009  |     105.329  |      95.087  |      109.593 |      99.3514 |     103.508  |      93.2663 |    104.    759     |     94.5171 |
    |     104.226  |      93.997  |     242.831  |     232.602  |     104.908  |      94.6794 |      109.173 |      98.9438 |     103.088  |      92.8587 |    104.    339     |     94.1096 |
    |     103.807  |      93.5889 |     242.412  |     232.194  |     104.49   |      94.2713 |      108.754 |      98.5357 |     102.669  |      92.4506 |    103.    92      |     93.7014 |
    |     103.39   |      93.1799 |     241.995  |     231.785  |     104.072  |      93.8624 |      108.337 |      98.1267 |     102.252  |      92.0417 |    103.    503     |     93.2925 |
    |     102.974  |      92.7701 |     241.579  |     231.375  |     103.657  |      93.4525 |      107.921 |      97.7169 |     101.836  |      91.6318 |    103.    087     |     92.8826 |
    |     102.56   |      92.3591 |     241.165  |     230.964  |     103.242  |      93.0415 |      107.507 |      97.3059 |     101.422  |      91.2208 |    102.    673     |     92.4716 |
    |     102.147  |      91.9467 |     240.752  |     230.552  |     102.829  |      92.6292 |      107.094 |      96.8936 |     101.008  |      90.8085 |    102.    259     |     92.0593 |
    |     101.734  |      91.5329 |     240.339  |     230.138  |     102.417  |      92.2153 |      106.681 |      96.4797 |     100.596  |      90.3946 |    101.    847     |     91.6454 |
    |     101.323  |      91.1173 |     239.928  |     229.722  |     102.005  |      91.7997 |      106.27  |      96.0641 |     100.184  |      89.979  |    101.    435     |     91.2299 |
    |     100.747  |      90.8645 |     239.352  |     229.469  |     101.429  |      91.5469 |      105.694 |      95.8113 |      99.6087 |      89.7262 |    100.    859     |     90.977  |
    |      99.777  |      91.004  |     238.382  |     229.609  |     100.459  |      91.6864 |      104.724 |      95.9508 |      98.6388 |      89.8657 |     99.    8896    |     91.1165 |
    |      98.8063 |      91.1422 |     237.411  |     229.747  |      99.4887 |      91.8246 |      103.753 |      96.089  |      97.668  |      90.0039 |     98.    9188    |     91.2547 |
    |     283.06   |     275.658  |      61.6651 |      54.2627 |     283.743  |     276.34   |      288.007 |     280.605  |     281.922  |     274.52   |    283.    173     |    275.77   |
    |     283.214  |     274.661  |      61.8191 |      53.2661 |     283.897  |     275.344  |      288.161 |     279.608  |     282.076  |     273.523  |    283.    327     |    274.774  |
    |     283.37   |     273.665  |      61.9745 |      52.2699 |     284.052  |     274.347  |      288.316 |     278.612  |     282.231  |     272.527  |    283.    482     |    273.778  |
    |     283.201  |     272.995  |      61.8056 |      51.5995 |     283.883  |     273.677  |      288.147 |     277.941  |     282.062  |     271.856  |    283.    313     |    273.107  |
    |     282.779  |     272.578  |      61.3842 |      51.1825 |     283.462  |     273.26   |      287.726 |     277.524  |     281.641  |     271.439  |    282.    892     |    272.69   |
    |     282.359  |     272.159  |      60.9644 |      50.7644 |     283.042  |     272.842  |      287.306 |     277.106  |     281.221  |     271.021  |    282.    472     |    272.272  |
    |     281.941  |     271.74   |      60.546  |      50.345  |     282.623  |     272.423  |      286.888 |     276.687  |     280.803  |     270.602  |    282.    054     |    271.853  |
    |     281.524  |     271.319  |      60.1287 |      49.9241 |     282.206  |     272.002  |      286.471 |     276.266  |     280.386  |     270.181  |    281.    636     |    271.432  |
    |     281.107  |     270.897  |      59.7123 |      49.5016 |     281.79   |     271.579  |      286.054 |     275.844  |     279.969  |     269.758  |    281.    22      |    271.009  |
    |     280.692  |     270.472  |      59.2967 |      49.0771 |     281.374  |     271.155  |      285.639 |     275.419  |     279.554  |     269.334  |    280.    804     |    270.585  |
    |     280.277  |     270.046  |      58.8815 |      48.6505 |     280.959  |     270.728  |      285.223 |     274.992  |     279.138  |     268.907  |    280.    389     |    270.158  |
    |     279.862  |     269.617  |      58.4666 |      48.2215 |     280.544  |     270.299  |      284.809 |     274.563  |     278.723  |     268.478  |    279.    974     |    269.729  |
    |     279.447  |     269.185  |      58.0518 |      47.7899 |     280.129  |     269.867  |      284.394 |     274.132  |     278.309  |     268.047  |    279.    56      |    269.298  |
    |     279.032  |     268.751  |      57.6369 |      47.3554 |     279.714  |     269.433  |      283.979 |     273.697  |     277.894  |     267.612  |    279.    145     |    268.863  |
    |     278.617  |     268.313  |      57.2216 |      46.9179 |     279.299  |     268.995  |      283.563 |     273.26   |     277.478  |     267.175  |    278.    729     |    268.426  |
    |     278.201  |     267.872  |      56.8057 |      46.4771 |     278.883  |     268.555  |      283.148 |     272.819  |     277.063  |     266.734  |    278.    313     |    267.985  |
    |     277.784  |     267.428  |      56.389  |      46.0327 |     278.467  |     268.11   |      282.731 |     272.375  |     276.646  |     266.29   |    277.    897     |    267.54   |
    |     277.366  |     266.98   |      55.9713 |      45.5846 |     278.049  |     267.662  |      282.313 |     271.926  |     276.228  |     265.841  |    277.    479     |    267.092  |
    |     276.947  |     266.527  |      55.5523 |      45.1323 |     277.63   |     267.21   |      281.894 |     271.474  |     275.809  |     265.389  |    277.    06      |    266.64   |
    |     276.527  |     266.071  |      55.1319 |      44.6758 |     277.209  |     266.753  |      281.474 |     271.018  |     275.389  |     264.933  |    276.    64      |    266.183  |
    |     276.105  |     265.61   |      54.7097 |      44.2147 |     276.787  |     266.292  |      281.052 |     270.557  |     274.967  |     264.472  |    276.    217     |    265.722  |
    |     275.681  |     265.144  |      54.2856 |      43.7488 |     276.363  |     265.826  |      280.628 |     270.091  |     274.542  |     264.006  |    275.    793     |    265.256  |
    |     275.255  |     264.673  |      53.8594 |      43.2778 |     275.937  |     265.355  |      280.201 |     269.62   |     274.116  |     263.535  |    275.    367     |    264.785  |
    |     274.826  |     264.196  |      53.4308 |      42.8013 |     275.508  |     264.879  |      279.773 |     269.143  |     273.688  |     263.058  |    274.    938     |    264.309  |
    |     274.395  |     263.714  |      52.9995 |      42.3192 |     275.077  |     264.397  |      279.341 |     268.661  |     273.256  |     262.576  |    274.    507     |    263.827  |
    |     273.96   |     263.226  |      52.5653 |      41.8311 |     274.643  |     263.909  |      278.907 |     268.173  |     272.822  |     262.088  |    274.    073     |    263.339  |
    |     273.523  |     262.732  |      52.1279 |      41.3366 |     274.205  |     263.414  |      278.47  |     267.679  |     272.385  |     261.593  |    273.    636     |    262.844  |
    |     273.082  |     262.231  |      51.6871 |      40.8356 |     273.765  |     262.913  |      278.029 |     267.178  |     271.944  |     261.092  |    273.    195     |    262.343  |
    |     272.638  |     261.723  |      51.2427 |      40.3276 |     273.32   |     262.405  |      277.585 |     266.67   |     271.5    |     260.584  |    272.    75      |    261.835  |
    |     272.189  |     261.207  |      50.7942 |      39.8123 |     272.872  |     261.89   |      277.136 |     266.154  |     271.051  |     260.069  |    272.    302     |    261.32   |
    |     271.737  |     260.685  |      50.3416 |      39.2894 |     272.419  |     261.367  |      276.683 |     265.631  |     270.598  |     259.546  |    271.    849     |    260.797  |
    |     271.279  |     260.154  |      49.8843 |      38.7585 |     271.962  |     260.836  |      276.226 |     265.1    |     270.141  |     259.015  |    271.    392     |    260.266  |
    |     270.817  |     259.614  |      49.4223 |      38.2193 |     271.5    |     260.297  |      275.764 |     264.561  |     269.679  |     258.476  |    270.    93      |    259.727  |
    |     270.35   |     259.066  |      48.9551 |      37.6712 |     271.033  |     259.749  |      275.297 |     264.013  |     269.212  |     257.928  |    270.    463     |    259.179  |
    |     269.878  |     258.509  |      48.4824 |      37.114  |     270.56   |     259.192  |      274.824 |     263.456  |     268.739  |     257.371  |    269.    99      |    258.622  |
    |     269.399  |     257.942  |      48.0039 |      36.5473 |     270.081  |     258.625  |      274.346 |     262.889  |     268.261  |     256.804  |    269.    512     |    258.055  |
    |     268.914  |     257.366  |      47.5193 |      35.9705 |     269.597  |     258.048  |      273.861 |     262.312  |     267.776  |     256.227  |    269.    027     |    257.478  |
    |     268.423  |     256.778  |      47.0282 |      35.3832 |     269.106  |     257.461  |      273.37  |     261.725  |     267.285  |     255.64   |    268.    536     |    256.891  |
    |     267.925  |     256.18   |      46.5301 |      34.785  |     268.608  |     256.863  |      272.872 |     261.127  |     266.787  |     255.042  |    268.    038     |    256.293  |
    |     267.42   |     255.571  |      46.0248 |      34.1754 |     268.102  |     256.253  |      272.367 |     260.517  |     266.282  |     254.432  |    267.    533     |    255.683  |
    |     266.907  |     254.949  |      45.5119 |      33.5539 |     267.589  |     255.631  |      271.854 |     259.896  |     265.769  |     253.811  |    267.    02      |    255.062  |
    |     266.386  |     254.315  |      44.9908 |      32.9199 |     267.068  |     254.997  |      271.333 |     259.262  |     265.248  |     253.177  |    266.    498     |    254.428  |
    |     265.856  |     253.668  |      44.4612 |      32.2728 |     266.539  |     254.35   |      270.803 |     258.615  |     264.718  |     252.53   |    265.    969     |    253.78   |
    |     265.318  |     253.007  |      43.9226 |      31.6122 |     266      |     253.69   |      270.265 |     257.954  |     264.179  |     251.869  |    265.    43      |    253.12   |
    |     264.77   |     252.332  |      43.3745 |      30.9373 |     265.452  |     253.015  |      269.716 |     257.279  |     263.631  |     251.194  |    264.    882     |    252.445  |
    |     264.212  |     251.643  |      42.8165 |      30.2476 |     264.894  |     252.325  |      269.158 |     256.589  |     263.073  |     250.504  |    264.    324     |    251.755  |
    |     263.643  |     250.937  |      42.2479 |      29.5424 |     264.325  |     251.62   |      268.59  |     255.884  |     262.505  |     249.799  |    263.    756     |    251.05   |
    |     263.063  |     250.216  |      41.6682 |      28.821  |     263.746  |     250.898  |      268.01  |     255.163  |     261.925  |     249.078  |    263.    176     |    250.329  |
    |     262.472  |     249.478  |      41.0769 |      28.0826 |     263.154  |     250.16   |      267.419 |     254.425  |     261.334  |     248.339  |    262.    585     |    249.59   |
    |     261.868  |     248.722  |      40.4733 |      27.3267 |     262.551  |     249.404  |      266.815 |     253.669  |     260.73   |     247.583  |    261.    981     |    248.834  |
    |     261.252  |     247.947  |      39.8568 |      26.5522 |     261.934  |     248.63   |      266.199 |     252.894  |     260.114  |     246.809  |    261.    364     |    248.06   |
    |     260.622  |     247.154  |      39.2267 |      25.7585 |     261.304  |     247.836  |      265.569 |     252.1    |     259.484  |     246.015  |    260.    734     |    247.266  |
    |     259.977  |     246.34   |      38.5823 |      24.9447 |     260.66   |     247.022  |      264.924 |     251.287  |     258.839  |     245.202  |    260.    09      |    246.452  |
    |     259.318  |     245.505  |      37.9229 |      24.1098 |     260      |     246.187  |      264.265 |     250.452  |     258.18   |     244.367  |    259.    431     |    245.618  |
    |     258.643  |     244.648  |      37.2476 |      23.2531 |     259.325  |     245.331  |      263.59  |     249.595  |     257.504  |     243.51   |    258.    755     |    244.761  |
    |     257.951  |     243.768  |      36.5557 |      22.3734 |     258.633  |     244.451  |      262.898 |     248.715  |     256.813  |     242.63   |    258.    063     |    243.881  |
    |     257.241  |     242.865  |      35.8463 |      21.4698 |     257.924  |     243.547  |      262.188 |     247.812  |     256.103  |     241.727  |    257.    354     |    242.977  |
    |     131.775  |     117.399  |     270.38   |     256.004  |     132.458  |     118.082  |      136.722 |     122.346  |     130.637  |     116.261  |    131.    888     |    117.512  |
    |     130.901  |     116.712  |     269.506  |     255.317  |     131.584  |     117.395  |      135.848 |     121.659  |     129.763  |     115.574  |    131.    014     |    116.825  |
    |     130.05   |     116.042  |     268.655  |     254.647  |     130.733  |     116.725  |      134.997 |     120.989  |     128.912  |     114.904  |    130.    163     |    116.155  |
    |     129.22   |     115.388  |     267.825  |     253.993  |     129.903  |     116.07   |      134.167 |     120.334  |     128.082  |     114.249  |    129.    333     |    115.5    |
    |     128.411  |     114.748  |     267.016  |     253.353  |     129.094  |     115.43   |      133.358 |     119.695  |     127.273  |     113.61   |    128.    524     |    114.861  |
    |     127.622  |     114.123  |     266.227  |     252.728  |     128.305  |     114.805  |      132.569 |     119.069  |     126.484  |     112.984  |    127.    735     |    114.235  |
    |     126.852  |     113.511  |     265.457  |     252.116  |     127.535  |     114.193  |      131.799 |     118.458  |     125.714  |     112.373  |    126.    965     |    113.623  |
    |     126.1    |     112.912  |     264.705  |     251.517  |     126.783  |     113.594  |      131.047 |     117.859  |     124.962  |     111.774  |    126.    213     |    113.024  |
    |     125.366  |     112.325  |     263.971  |     250.93   |     126.048  |     113.007  |      130.313 |     117.272  |     124.228  |     111.187  |    125.    478     |    112.438  |
    |     124.648  |     111.75   |     263.253  |     250.355  |     125.33   |     112.432  |      129.595 |     116.697  |     123.51   |     110.612  |    124.    761     |    111.862  |
    |     123.946  |     111.186  |     262.551  |     249.791  |     124.629  |     111.868  |      128.893 |     116.133  |     122.808  |     110.047  |    124.    059     |    111.298  |
    |     123.26   |     110.632  |     261.865  |     249.237  |     123.942  |     111.314  |      128.207 |     115.579  |     122.121  |     109.494  |    123.    372     |    110.745  |
    |     122.588  |     110.088  |     261.193  |     248.693  |     123.27   |     110.771  |      127.535 |     115.035  |     121.45   |     108.95   |    122.    7       |    110.201  |
    |     121.93   |     109.554  |     260.535  |     248.159  |     122.612  |     110.236  |      126.877 |     114.501  |     120.792  |     108.416  |    122.    042     |    109.667  |
    |     121.285  |     109.029  |     259.89   |     247.634  |     121.968  |     109.711  |      126.232 |     113.976  |     120.147  |     107.891  |    121.    398     |    109.141  |
    |     120.654  |     108.512  |     259.259  |     247.117  |     121.336  |     109.194  |      125.601 |     113.459  |     119.515  |     107.374  |    120.    766     |    108.625  |
    |     120.034  |     108.003  |     258.639  |     246.608  |     120.717  |     108.686  |      124.981 |     112.95   |     118.896  |     106.865  |    120.    147     |    108.116  |
    |     119.427  |     107.502  |     258.032  |     246.107  |     120.109  |     108.185  |      124.374 |     112.449  |     118.289  |     106.364  |    119.    539     |    107.615  |
    |     118.83   |     107.009  |     257.435  |     245.613  |     119.513  |     107.691  |      123.777 |     111.955  |     117.692  |     105.87   |    118.    943     |    107.121  |
    |     118.245  |     106.522  |     256.85   |     245.126  |     118.927  |     107.204  |      123.192 |     111.468  |     117.107  |     105.383  |    118.    357     |    106.634  |
    |     117.67   |     106.041  |     256.274  |     244.646  |     118.352  |     106.724  |      122.616 |     110.988  |     116.531  |     104.903  |    117.    782     |    106.154  |
    |     117.104  |     105.567  |     255.709  |     244.172  |     117.786  |     106.249  |      122.051 |     110.514  |     115.966  |     104.429  |    117.    217     |    105.68   |
    |     116.548  |     105.099  |     255.153  |     243.703  |     117.23   |     105.781  |      121.495 |     110.045  |     115.41   |     103.96   |    116.    661     |    105.211  |
    |     116.001  |     104.636  |     254.606  |     243.24   |     116.683  |     105.318  |      120.948 |     109.582  |     114.863  |     103.497  |    116.    114     |    104.748  |
    |     115.463  |     104.178  |     254.068  |     242.783  |     116.145  |     104.86   |      120.409 |     109.125  |     114.324  |     103.04   |    115.    575     |    104.29   |
    |     114.932  |     103.725  |     253.537  |     242.33   |     115.615  |     104.407  |      119.879 |     108.672  |     113.794  |     102.587  |    115.    045     |    103.837  |
    |     114.41   |     103.277  |     253.015  |     241.881  |     115.093  |     103.959  |      119.357 |     108.223  |     113.272  |     102.138  |    114.    523     |    103.389  |
    |     113.895  |     102.832  |     252.5    |     241.437  |     114.578  |     103.515  |      118.842 |     107.779  |     112.757  |     101.694  |    114.    008     |    102.945  |
    |     113.388  |     102.392  |     251.993  |     240.997  |     114.07   |     103.075  |      118.335 |     107.339  |     112.25   |     101.254  |    113.    5       |    102.505  |
    |     112.887  |     101.956  |     251.492  |     240.561  |     113.569  |     102.639  |      117.834 |     106.903  |     111.749  |     100.818  |        113         |    102.069  |
    |     112.393  |     101.523  |     250.998  |     240.128  |     113.075  |     102.206  |      117.34  |     106.47   |     111.254  |     100.385  |    112.    505     |    101.636  |
    |     111.905  |     101.094  |     250.51   |     239.699  |     112.587  |     101.776  |      116.852 |     106.041  |     110.766  |      99.9555 |    112.    017     |    101.206  |
    |     111.423  |     100.667  |     250.028  |     239.272  |     112.105  |     101.35   |      116.369 |     105.614  |     110.284  |      99.5289 |    111.    535     |    100.78   |
    |     110.946  |     100.243  |     249.551  |     238.848  |     111.629  |     100.926  |      115.893 |     105.19   |     109.808  |      99.1051 |    111.    059     |    100.356  |
    |     110.475  |      99.8221 |     249.08   |     238.427  |     111.157  |     100.505  |      115.422 |     104.769  |     109.337  |      98.6838 |    110.    588     |     99.9347 |
    |     110.009  |      99.4031 |     248.614  |     238.008  |     110.691  |     100.086  |      114.956 |     104.35   |     108.871  |      98.2649 |    110.    122     |     99.5157 |
    |     109.548  |      98.9863 |     248.153  |     237.591  |     110.23   |      99.6687 |      114.495 |     103.933  |     108.41   |      97.848  |    109.    66      |     99.0988 |
    |     109.091  |      98.5713 |     247.696  |     237.176  |     109.774  |      99.2537 |      114.038 |     103.518  |     107.953  |      97.433  |    109.    204     |     98.6838 |
    |     108.639  |      98.1579 |     247.244  |     236.763  |     109.321  |      98.8403 |      113.586 |     103.105  |     107.501  |      97.0196 |    108.    751     |     98.2704 |
    |     108.191  |      97.746  |     246.796  |     236.351  |     108.873  |      98.4284 |      113.137 |     102.693  |     107.052  |      96.6077 |    108.    303     |     97.8585 |
    |     107.746  |      97.3353 |     246.351  |     235.94   |     108.429  |      98.0178 |      112.693 |     102.282  |     106.608  |      96.1971 |    107.    859     |     97.4479 |
    |     107.305  |      96.9257 |     245.91   |     235.531  |     107.988  |      97.6081 |      112.252 |     101.873  |     106.167  |      95.7875 |    107.    418     |     97.0383 |
    |     106.868  |      96.517  |     245.473  |     235.122  |     107.551  |      97.1994 |      111.815 |     101.464  |     105.73   |      95.3787 |    106.    981     |     96.6295 |
    |     106.434  |      96.1088 |     245.039  |     234.714  |     107.116  |      96.7913 |      111.381 |     101.056  |     105.296  |      94.9706 |    106.    546     |     96.2214 |
    |     106.003  |      95.7012 |     244.608  |     234.306  |     106.685  |      96.3836 |      110.949 |     100.648  |     104.864  |      94.5629 |    106.    115     |     95.8137 |
    |     105.574  |      95.2938 |     244.179  |     233.899  |     106.257  |      95.9762 |      110.521 |     100.241  |     104.436  |      94.1555 |    105.    687     |     95.4064 |
    |     105.148  |      94.8865 |     243.753  |     233.491  |     105.831  |      95.5689 |      110.095 |      99.8333 |     104.01   |      93.7482 |    105.    261     |     94.999  |
    |     104.725  |      94.4791 |     243.33   |     233.084  |     105.407  |      95.1615 |      109.672 |      99.4259 |     103.586  |      93.3408 |    104.    837     |     94.5916 |
    |     104.303  |      94.0713 |     242.908  |     232.676  |     104.986  |      94.7537 |      109.25  |      99.0181 |     103.165  |      92.9331 |    104.    416     |     94.1839 |
    |     103.884  |      93.6631 |     242.489  |     232.268  |     104.566  |      94.3455 |      108.831 |      98.6099 |     102.746  |      92.5248 |    103.    996     |     93.7756 |
    |     103.466  |      93.2542 |     242.071  |     231.859  |     104.148  |      93.9366 |      108.413 |      98.201  |     102.328  |      92.1159 |    103.    579     |     93.3667 |
    |     103.05   |      92.8444 |     241.655  |     231.449  |     103.732  |      93.5268 |      107.997 |      97.7912 |     101.912  |      91.7061 |    103.    162     |     92.9569 |
    |     102.635  |      92.4335 |     241.24   |     231.038  |     103.317  |      93.1159 |      107.582 |      97.3803 |     101.497  |      91.2952 |    102.    748     |     92.5461 |
    |     102.221  |      92.0214 |     240.826  |     230.626  |     102.904  |      92.7038 |      107.168 |      96.9682 |     101.083  |      90.8831 |    102.    334     |     92.134  |
    |     101.809  |      91.6078 |     240.414  |     230.213  |     102.491  |      92.2903 |      106.756 |      96.5546 |     100.671  |      90.4696 |    101.    921     |     91.7204 |
    |     101.397  |      91.1926 |     240.002  |     229.798  |     102.079  |      91.8751 |      106.344 |      96.1394 |     100.259  |      90.0544 |    101.    51      |     91.3052 |
    |     100.947  |      90.8142 |     239.552  |     229.419  |     101.63   |      91.4966 |      105.894 |      95.761  |      99.8089 |      89.6759 |    101.    06      |     90.9267 |
    |      99.9762 |      90.9553 |     238.581  |     229.56   |     100.659  |      91.6378 |      104.923 |      95.9021 |      98.8379 |      89.8171 |    100.    089     |     91.0679 |
    |      99.0046 |      91.0951 |     237.609  |     229.7    |      99.687  |      91.7775 |      103.951 |      96.0419 |      97.8663 |      89.9569 |     99.    1171    |     91.2077 |
    +--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------+--------------    +-------------   +-------------+


# API

The JWST GTVT API allows you to break down and interact with the different components available in the code base:

    $ python
    >>> from jwst_gtvt.jwst_tvt import Ephemeris
    >>> eph = Ephemeris()
    >>> eph.display_ephemeris_header()

    '*******************************************************************************'
    'Ephemeris / WWW_USER DOW Month Day Time Year      Pasadena, USA      / Horizons'
    '*******************************************************************************'
    'Target body name: James Webb Space Telescope (spacecraft) (-170) {source: JWST_merged}'
    'Center body name: Sun (10)                        {source: DE441}'
    'Center-site name: BODY CENTER'
    '*******************************************************************************'
    'Start time      : A.D. 2021-Dec-26 00:00:00.0000 TDB'
    'Stop  time      : A.D. 2024-Oct-03 00:00:00.0000 TDB'
    'Step-size       : 1440 minutes'
    '*******************************************************************************'
    'Center geodetic : 0.0, 0.0, 0.0                   {E-lon(deg),Lat(deg),Alt(km)}'
    'Center cylindric: 0.0, 0.0, 0.0                   {E-lon(deg),Dxy(km),Dz(km)}'
    'Center radii    : 696000.0, 696000.0, 696000.0 km {Equator_a, b, pole_c}'
    'Output units    : KM-S' 'Calendar mode   : Mixed Julian/Gregorian'
    'Output type     : LT CORRECTED cartesian states'
    'Output format   : 2 (position and velocity)' 'Reference frame : ICRF'
    '*******************************************************************************']

    >>> eph.display_ephemeris_footer()

    ['**************************************************************************************************************************************************************************************************'
    ' ' 'TIME' ''
    '  Barycentric Dynamical Time ("TDB" or T_eph) output was requested. This'
    'continuous coordinate time is equivalent to the relativistic proper time'
    'of a clock at rest in a reference frame co-moving with the solar system'
    "barycenter but outside the system's gravity well. It is the independent"
    'variable in the solar system relativistic equations of motion.' ''
    '  TDB runs at a uniform rate of one SI second per second and is independent'
    "of irregularities in Earth's rotation." ' ' 'CALENDAR SYSTEM' ''
    '  Mixed calendar mode was active such that calendar dates after AD 1582-Oct-15'
    '(if any) are in the modern Gregorian system. Dates prior to 1582-Oct-5 (if any)'
    'are in the Julian calendar system, which is automatically extended for dates'
    'prior to its adoption on 45-Jan-1 BC.  The Julian calendar is useful for'
    'matching historical dates. The Gregorian calendar more accurately corresponds'
    'to the Earth\'s orbital motion and seasons. A "Gregorian-only" calendar mode is'
    'available if such physical events are the primary interest.' ''
    'REFERENCE FRAME AND COORDINATES' ''
    '  International Celestial Reference Frame (ICRF)' ''
    '    The ICRF is an adopted reference frame whose axes are defined relative to '
    '    fixed extragalactic radio sources distributed across the sky.' ''
    '    The ICRF was aligned with the prior FK5/J2000 dynamical system at the ~0.02 '
    '    arcsecond level but is not identical and has no associated standard epoch.'
    '' '  Symbol meaning:' ''
    '    JDTDB    Julian Day Number, Barycentric Dynamical Time'
    '      X      X-component of position vector (km)'
    '      Y      Y-component of position vector (km)'
    '      Z      Z-component of position vector (km)'
    '      VX     X-component of velocity vector (km/sec)                           '
    '      VY     Y-component of velocity vector (km/sec)                           '
    '      VZ     Z-component of velocity vector (km/sec)                           '
    '' 'ABERRATIONS AND CORRECTIONS' ''
    ' States have been corrected for Newtonian down-leg (receive) light-time'
    ' displacement.' '' 'Computations by ...' ''
    '    Solar System Dynamics Group, Horizons On-Line Ephemeris System'
    '    4800 Oak Grove Drive, Jet Propulsion Laboratory'
    '    Pasadena, CA  91109   USA' ''
    '    General site: https://ssd.jpl.nasa.gov/'
    '    Mailing list: https://ssd.jpl.nasa.gov/email_list.html'
    '    System news : https://ssd.jpl.nasa.gov/horizons/news.html'
    '    User Guide  : https://ssd.jpl.nasa.gov/horizons/manual.html'
    '    Connect     : browser        https://ssd.jpl.nasa.gov/horizons/app.html#/x'
    '                  API            https://ssd-api.jpl.nasa.gov/doc/horizons.html'
    '                  command-line   telnet ssd.jpl.nasa.gov 6775'
    '                  e-mail/batch   https://ssd.jpl.nasa.gov/ftp/ssd/hrzn_batch.txt'
    '                  scripts        https://ssd.jpl.nasa.gov/ftp/ssd/SCRIPTS'
    '    Author      : Jon.D.Giorgini@jpl.nasa.gov'
    '*******************************************************************************'
    '' '!$$SOF' 'COMMAND = -170' "OBJ_DATA = 'NO'" 'EPHEM_TYPE = VECTORS'
    "START_TIME = '2021-12-26'" "STOP_TIME = '2024-10-03'"
    "CENTER = '500@10'" "STEP_SIZE = '1 DAYS'" "CSV_FORMAT = 'YES'"
    "VEC_TABLE = '2'" "REF_SYSTEM = 'ICRF'" "REF_PLANE = 'FRAME'"
    "VEC_CORR = 'LT'" "OUT_UNITS = 'KM-S'" "VEC_LABELS = 'YES'"
    "VEC_DELTA_T = 'NO'"]

    >>> eph.dataframe

            JDTDB              Calendar Date (TDB)             X             Y             Z      MJD
    0     2459574.5   A.D. 2021-Dec-26 00:00:00.0000           NaN           NaN           NaN  59574.0
    1     2459575.5   A.D. 2021-Dec-27 00:00:00.0000 -1.051333e+07  1.347734e+08  5.836535e+07  59575.0
    2     2459576.5   A.D. 2021-Dec-28 00:00:00.0000 -1.313013e+07  1.347285e+08  5.828041e+07  59576.0
    3     2459577.5   A.D. 2021-Dec-29 00:00:00.0000 -1.574720e+07  1.345992e+08  5.817769e+07  59577.0
    4     2459578.5   A.D. 2021-Dec-30 00:00:00.0000 -1.835999e+07  1.344084e+08  5.805701e+07  59578.0
    ...         ...                              ...           ...           ...           ...      ...
    1008  2460582.5   A.D. 2024-Sep-29 00:00:00.0000  1.504994e+08  1.211053e+07  5.587302e+06  60582.0
    1009  2460583.5   A.D. 2024-Sep-30 00:00:00.0000  1.502033e+08  1.449748e+07  6.621077e+06  60583.0
    1010  2460584.5   A.D. 2024-Oct-01 00:00:00.0000  1.498622e+08  1.688006e+07  7.652313e+06  60584.0
    1011  2460585.5   A.D. 2024-Oct-02 00:00:00.0000  1.494761e+08  1.925750e+07  8.680688e+06  60585.0
    1012  2460586.5   A.D. 2024-Oct-03 00:00:00.0000  1.490451e+08  2.162904e+07  9.705883e+06  60586.0

    >>> ceres_df = eph.get_moving_target_positions('Ceres')
    >>> from jwst_gtvt.plotting import plot_visibility
    >>> plot_visibility(eph)


# Troubleshooting

Unless specified, astroquery will cache queries to Horizons. This can cause some issues when trying to query targets
when changes are made upstream at JPL. To find and clear the cache, you can use astropy:

    >>> import astropy
    >>> path = astropy.config.get_cache_dir()
    >>> print(path)

This will print where astropy is caching all of its data. If astroquery has been used to search for targets in the past,
`<value_of_path>/astroquery/Horizons/` will exist. Now you have the full path and can remove it manually.

You can also use our `delete_cache` utility function. After the software is installed you can enter:
   
    $ delete_cache

If cached results exist you will be prompted whether you want to delete the cache folder or not.


# Contributing

The `jwst_gtvt` follows the STScI ["forking workflow"](https://github.com/spacetelescope/style-guides/blob/master/guides/git-workflow.md#forking-workflow).


# Citation

If you use the JWST General Target Visibility Tool (GTVT) or JWST Moving Target Visibility Tool (MTVT) for work/research presented in a publication, or as a dependency in a software package, we recommend and encourage the following acknowledgment:

This work made use of the JWST General Target Visibility Tool (GTVT) and/or the JWST Moving Target Visibility Tool (MTVT) originally written by Matthew Hill, expanded for moving targets by Michael Kelley, and maintained by Mees Fix.
